@@ -110,6 +110,26 @@
 //!
 //! ### Features
 //!
+//! #### `nightly`
+//!
+//! Uses [Macro MetaVar Expression Counting](https://rust-lang.github.io/rfcs/3086-macro-metavar-expr.html#count) instead of
+//! [Repetition Counting through tuple slice length](https://danielkeep.github.io/tlborm/book/blk-counting.html#slice-length).
+//!
+//! Enabling nightly feature flag does not have any impact on the performance, neither runtime nor compile time,
+//! since both stable and nightly approach gets optimized and inlined at compile time, but has the same
+//! negligible compile-time cost.
+//!
+//! This only exists because [Macro MetaVar Expression](https://rust-lang.github.io/rfcs/3086-macro-metavar-expr.html)
+//! is still being discussed[^1] (and may not reach stable 1.63 or 1.64, and it is late for 1.62),
+//! even though `$$` and `${ignore(_)}` are targeting 1.62[^2] and
+//! [it will probably be delivered](https://github.com/rust-lang/rust/pull/95860#issuecomment-1094136943),
+//! `${count(_)}` is one of the features that are left out of the stabilization because it may need more
+//! refinement and discussion
+//!
+//! **bindet** don't need this *personally*, since it does not have too much elements to cause a compiler crash,
+//! but keeping it in the source code helps us to remember to deliver it as a default when it reach stable,
+//! and reduce the amount of hacky/tricky things we need to do with declarative macros (which already needs a bunch of tricks).
+//!
 //! #### `mime`
 //! Enables conversion from [`FileType`][`types::FileType`] and [`FileRootType`][`FileRootType`] to
 //! `Mime` by implementing `TryInto<Mime>` trait,
@@ -121,12 +141,17 @@
 //! `MediaTypeBuf` by implementing `TryInto<MediaTypeBuf>` trait,
 //! there is no need to `use` any additional module, just enable the feature.
 //!
-#![feature(test)]
+//! [^1]: <https://github.com/rust-lang/rust/issues/83527>
+//!
+//! [^2]: <https://github.com/rust-lang/rust/pull/95860>
+#![cfg_attr(feature = "nightly", feature(test))]
+// TODO: looking for https://github.com/rust-lang/rust/issues/83527
+#![cfg_attr(feature = "nightly", feature(macro_metavar_expr))]
 use crate::description::FileTypeDescription;
 use crate::matcher::{FileTypeMatcher, RelativePosition, Step, TestResult};
 pub use crate::types::FileRootType;
 pub use crate::types::FileType;
-use std::io::{Read, Seek, SeekFrom};
+use std::io::{ErrorKind, Read, Seek, SeekFrom};
 use std::prelude::rust_2021::TryFrom;
 
 #[cfg(any(feature = "mime", feature = "mediatype"))]
@@ -180,7 +205,7 @@ where
     R: Read,
     R: Seek,
 {
-    let at_start = detect_at_start(&mut read)?;
+    let at_start = detect_at_start_from_ref(&mut read)?;
 
     if let Some(start) = at_start {
         let types: Vec<FileType> = start
@@ -188,7 +213,7 @@ where
             .iter()
             .map(|s| s.file_type.clone())
             .collect();
-        let at_end = detect_at_end_with_(&mut read, types.as_slice())?;
+        let at_end = detect_variants_at_end_from_ref(&mut read, types.as_slice())?;
 
         if let Some(at_end) = at_end {
             let start_matches: Vec<FileType> =
@@ -237,30 +262,93 @@ where
 /// may be reported with [`FileTypeMatch.full_match = false`](FileTypeMatch) signaling a probable
 /// match.
 ///
-/// This is a less reliable version of [`detect`], but with guaranteed `O(1)` time complexity,
-/// in other words, it takes the same time to check small files and larger files.
+/// This is a less reliable version of [`detect`], but with amortized `O(1)` time complexity.
 ///
 /// Since different [FileType] detection implementations receives the same data slice, it may produce
 /// more than one matching types.
-pub fn detect_at_start<R>(read: &mut R) -> Result<Option<FileTypeMatches>, std::io::Error>
+///
+/// This version receives a `variants` parameter that allows to specify which file types to check against.
+///
+/// **This is the ownership taking version of [`detect_at_start_from_ref`].**
+///
+/// Read more in [`detect_variants_at_start_from_ref`].
+pub fn detect_at_start<R>(mut read: R) -> Result<Option<FileTypeMatches>, std::io::Error>
 where
     R: Read,
 {
-    return detect_at_start_with_(read, &FileType::variants());
+    return detect_variants_at_start_from_ref(&mut read, &FileType::variants());
 }
 
 /// Detect a file type by looking at the start of the file. Types that need a second check at the end
 /// may be reported with [`FileTypeMatch.full_match = false`](FileTypeMatch) signaling a probable
 /// match.
 ///
-/// This is a less reliable version of [`detect`], but with guaranteed `O(1)` time complexity,
-/// in other words, it takes the same time to check small files and larger files.
+/// This is a less reliable version of [`detect`], but with amortized `O(1)` time complexity.
+///
+/// Since different [FileType] detection implementations receives the same data slice, it may produce
+/// more than one matching types.
+///
+/// This version receives a `variants` parameter that allows to specify which file types to check against.
+///
+/// **This is the reference taking version of [`detect_at_start`].**
+///
+/// Read more in [`detect_variants_at_start_from_ref`].
+pub fn detect_at_start_from_ref<R>(read: &mut R) -> Result<Option<FileTypeMatches>, std::io::Error>
+where
+    R: Read,
+{
+    return detect_variants_at_start_from_ref(read, &FileType::variants());
+}
+
+/// Detect a file type by looking at the start of the file. Types that need a second check at the end
+/// may be reported with [`FileTypeMatch.full_match = false`](FileTypeMatch) signaling a probable
+/// match.
+///
+/// This is a less reliable version of [`detect`], but with amortized `O(1)` time complexity.
+///
+/// Since different [FileType] detection implementations receives the same data slice, it may produce
+/// more than one matching types.
+///
+/// This version receives a `variants` parameter that allows to specify which file types to check against.
+///
+/// **This is the ownership taking version of [`detect_variants_at_start_from_ref`].**
+///
+/// Read more in [`detect_variants_at_start_from_ref`].
+pub fn detect_variants_at_start<R>(
+    mut read: R,
+    variants: &[FileType],
+) -> Result<Option<FileTypeMatches>, std::io::Error>
+where
+    R: Read,
+{
+    return detect_variants_at_start_from_ref(&mut read, variants);
+}
+
+/// Detect a file type by looking at the start of the file. Types that need a second check at the end
+/// may be reported with [`FileTypeMatch.full_match = false`](FileTypeMatch) signaling a probable
+/// match.
+///
+/// This is a less reliable version of [`detect`], but with amortized `O(1)` time complexity.
 ///
 /// Since different [FileType] detection implementations receives the same data slice, it may produce
 /// more than one matching types.
 ///
 /// This version receives a `variants` parameter that allows to specify which file types to check.
-pub fn detect_at_start_with_<R>(
+///
+/// **This is the reference taking version.**
+///
+/// ## Performance note
+///
+/// It is not `O(1)` time complexity because small files has less variants to test against, and bigger
+/// files has more variants, also, bigger files are slower to read (even if you read the same amount of bytes),
+/// mainly on compressed File Systems. In addition to this, files that match perfectly any variant
+/// (i.e. are detected) have faster detection time than files that does not match, this include bigger files,
+/// for example, an 1GiB `AVI` can be detected by looking at the first 4 bytes, while a 100MiB that is
+/// not detectable by this lib, falls to the second detection approach, which reads more data from the file.
+///
+/// The worst performance is always the [maximum block size of provided `variants`](FileType::maximum_block_size_of_variants),
+/// so any undetectable file bigger than this will have the same performance regardless of its size.
+pub fn detect_variants_at_start_from_ref<R>(
     read: &mut R,
     variants: &[FileType],
 ) -> Result<Option<FileTypeMatches>, std::io::Error>
@@ -269,18 +357,26 @@ where
 {
     let start_position = RelativePosition::Start;
     let small = FileType::ideal_block_size_of_variants(&start_position, variants);
-
     let mut matches: Vec<FileTypeMatch> = vec![];
 
-    let mut readed_: Vec<u8> = vec![];
+    let mut read_data: Vec<u8> = vec![];
 
     if let Some((size, types)) = small {
         let mut buff = vec![0u8; size];
         let buff_slice = &mut buff[..];
 
-        let read = read.read(buff_slice)?;
+        let read = loop {
+            // Do this because Read::read may fail on interruption, but it does not mean that there is no data to read.
+            // This is mostly the same approach the `Read::read_to_end` uses.
+            match read.read(buff_slice) {
+                Ok(i) => break i,
+                Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+                Err(e) => return Err(e),
+            };
+        };
+
         let bytes = &buff_slice[..read];
-        readed_.extend_from_slice(&bytes);
+        read_data.extend_from_slice(&bytes);
 
         push_matched_types_into(&mut matches, &bytes, &start_position, &Step::Small, &types);
     }
@@ -294,36 +390,46 @@ where
             .map(|v| v.file_type.clone())
             .collect();
 
-        return Result::Ok(Some(FileTypeMatches {
+        return Ok(Some(FileTypeMatches {
             likely_to_be: perfect,
             all_matches: matches,
         }));
     }
 
-    let big = FileType::maximum_block_size(&start_position);
-
+    // Most of the time is taken here,
+    // this will change with the implementation of https://gitlab.com/Kores/bindet/-/issues/5
+    let big = FileType::maximum_block_size_of_variants(&start_position, &variants);
     if let Some((size, types)) = big {
-        let new_size = size - readed_.len();
+        // the size corresponding to remaining data that was not read yet.
+        // which is the maximum block SIZE minus the total of data that was already read.
+        let new_size = size - read_data.len();
+
+        // creates a buffer with the needed size to hold the remaining data.
         let mut buff = vec![0u8; new_size];
         let buff_slice = &mut buff[..];
-        let mut readed = read.read(buff_slice)?;
-        let mut all_readed = readed;
-        loop {
-            if readed != 0 && readed < size {
-                if let Ok(ok) = read.read(&mut buff_slice[all_readed..]) {
-                    readed = ok;
-                    all_readed += readed;
-                } else {
-                    break;
-                }
-            } else {
+        // stores the total amount of data read so far, including the data already read before.
+        let mut all_read = read_data.len();
+        // stores the amount of data that was filled to buff.
+        let mut filled = 0usize;
+
+        while all_read < new_size {
+            let read = match read.read(&mut buff_slice[filled..]) {
+                Ok(i) => i,
+                Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+                Err(e) => return Err(e),
+            };
+
+            if read == 0 {
                 break;
             }
+
+            filled += read;
+            all_read += read;
         }
 
-        if all_readed != 0 {
-            readed_.extend_from_slice(&buff_slice[..all_readed]);
-            let bytes = &readed_[..];
+        if filled != 0 {
+            read_data.extend_from_slice(&buff_slice[..filled]);
+            let bytes = &read_data[..];
 
             push_matched_types_into(&mut matches, &bytes, &start_position, &Step::Large, &types);
         }
@@ -331,13 +437,13 @@ where
 
     if !matches.is_empty() {
         let types: Vec<FileType> = matches.iter().map(|v| v.file_type.clone()).collect();
-        return Result::Ok(Some(FileTypeMatches {
+        return Ok(Some(FileTypeMatches {
             likely_to_be: types,
             all_matches: matches,
         }));
     }
 
-    Result::Ok(None)
+    Ok(None)
 }
 
 /// Detect a file type by using a backward sliding window, this approach does have a `O(n)` time complexity
@@ -347,12 +453,51 @@ where
 ///
 /// Since different [FileType] detection implementations receives the same data slice, it may produce
 /// more than one matching types.
-pub fn detect_at_end<R>(read: &mut R) -> Result<Option<FileTypeMatches>, std::io::Error>
+///
+/// **This is the ownership taking version of [detect_at_end_from_ref]**.
+pub fn detect_at_end<R>(mut read: R) -> Result<Option<FileTypeMatches>, std::io::Error>
 where
     R: Read,
     R: Seek,
 {
-    detect_at_end_with_(read, &FileType::variants())
+    detect_variants_at_end_from_ref(&mut read, &FileType::variants())
+}
+
+/// Detect a file type by using a backward sliding window, this approach does have a `O(n)` time complexity
+/// and is not meant to be used directly.
+///
+/// Currently this only works for [FileType::Zip].
+///
+/// Since different [FileType] detection implementations receives the same data slice, it may produce
+/// more than one matching types.
+///
+/// **This is the ownership taking version of [detect_variants_at_end_from_ref]**.
+pub fn detect_variants_at_end<R>(
+    mut read: R,
+    variants: &[FileType],
+) -> Result<Option<FileTypeMatches>, std::io::Error>
+where
+    R: Read,
+    R: Seek,
+{
+    detect_variants_at_end_from_ref(&mut read, variants)
+}
+
+/// Detect a file type by using a backward sliding window, this approach does have a `O(n)` time complexity
+/// and is not meant to be used directly.
+///
+/// Currently this only works for [FileType::Zip].
+///
+/// Since different [FileType] detection implementations receives the same data slice, it may produce
+/// more than one matching types.
+///
+/// **This is the reference taking version of [detect_at_end]**.
+pub fn detect_at_end_from_ref<R>(read: &mut R) -> Result<Option<FileTypeMatches>, std::io::Error>
+where
+    R: Read,
+    R: Seek,
+{
+    detect_variants_at_end_from_ref(read, &FileType::variants())
 }
 
 /// Detect a file type by using a backward sliding window, this approach does have a `O(n)` time complexity
@@ -364,7 +509,28 @@ where
 /// more than one matching types.
 ///
 /// This version receives a `variants` parameter that allows to specify which file types to check.
-pub fn detect_at_end_with_<R>(
+///
+/// **Maintenance Note**
+///
+/// Review this algorithm, backward sliding window should be an option, and file type detectors
+/// should signal to take them out of the detection list once a detection fails,
+/// so it stops wasting time when a detection fails.
+///
+/// Some detections may need a backward sliding window, others may only need to do an exact match
+/// in the bytes that appears at the end of the file.
+///
+/// This thing is also a consideration for the Milestone 1.0.0 release issues, both Basic, StartScan
+/// and FullScan should be reviewed, some file types may need a sliding window right from the start,
+/// which we don't support yet, and the workaround is to request the maximum possible size and do the
+/// sliding window detection from here, but does not work when the file is shorter than the requested size,
+/// also, some may signal to stop at a specific point, for example, the magic number may appear anywhere
+/// in between byte `(0..1000000)`, but not after this point, the same for backward sliding window.
+///
+/// And the other thing for consideration is arbitrary seeking, which could be a different `DetectionMode`,
+/// it would look at fixed positions of the file for the magic number, and it would be good to be able
+/// to create a sliding window from here to the start or to the end, with a maximum size support,
+/// or undefined size, which would be the same as sliding through the entire file.
+pub fn detect_variants_at_end_from_ref<R>(
     read: &mut R,
     variants: &[FileType],
 ) -> Result<Option<FileTypeMatches>, std::io::Error>
@@ -394,7 +560,16 @@ where
 
         loop {
             if seek > 0 {
-                let read_bytes = read.read(buff_slice)?;
+                // Do this because Read::read may fail on interruption, but it does not mean that there is no data to read.
+                // This is mostly the same approach the `Read::read_to_end` uses.
+                let read_bytes = loop {
+                    match read.read(buff_slice) {
+                        Ok(i) => break i,
+                        Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+                        Err(e) => return Err(e),
+                    };
+                };
+
                 let bytes = &buff_slice[..read_bytes];
 
                 push_matched_types_into(&mut matches, &bytes, &end_position, &Step::Small, &types);
@@ -419,14 +594,14 @@ where
             .map(|v| v.file_type.clone())
             .collect();
 
-        return Result::Ok(Some(FileTypeMatches {
+        return Ok(Some(FileTypeMatches {
             likely_to_be: perfect,
             all_matches: matches,
         }));
     }
 
     // TODO: none of the FileTypes does have a maximum_block_size for windowing
-    /*let big = FileType::maximum_block_size(&end_position);
+    /*let big = FileType::maximum_block_size_of_variants(&end_position, variants);
 
     if let Some((size, types)) = big {
         let new_size = size - readed_.len();
@@ -444,13 +619,13 @@ where
 
     if !matches.is_empty() {
         let types: Vec<FileType> = matches.iter().map(|v| v.file_type.clone()).collect();
-        return Result::Ok(Some(FileTypeMatches {
+        return Ok(Some(FileTypeMatches {
             likely_to_be: types,
             all_matches: matches,
         }));
     }
 
-    Result::Ok(None)
+    Ok(None)
 }
 
 fn push_matched_types_into(
@@ -473,14 +648,14 @@ fn push_matched_types_into(
 
 #[cfg(test)]
 mod tests {
-    extern crate test;
     use crate::types::FileType;
-    use crate::{detect, detect_at_end, FileTypeMatch, FileTypeMatches};
+    use crate::{
+        detect, detect_at_end_from_ref, detect_at_start_from_ref, FileTypeMatch, FileTypeMatches,
+    };
     use std::collections::HashSet;
     use std::fs::OpenOptions;
     use std::io::{BufReader, Error, ErrorKind};
     use std::path::Path;
-    use test::Bencher;
 
     #[test]
     fn test_zip_detect() {
@@ -503,6 +678,28 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_fast_rar_sfx_detect() {
+        test_fast_detect_match_n(
+            "files/hello-world.exe",
+            vec![
+                FileTypeMatch::new(FileType::Rar5, true),
+                FileTypeMatch::new(FileType::DosMzExecutable, false),
+            ],
+        );
+    }
+
+    #[test]
+    fn test_2mib_rar_sfx_detect() {
+        test_fast_detect_match_n(
+            "files/2mib.exe",
+            vec![
+                FileTypeMatch::new(FileType::Rar5, true),
+                FileTypeMatch::new(FileType::DosMzExecutable, false),
+            ],
+        );
+    }
+
     #[test]
     fn test_png_detect() {
         test_detect_match("files/rust-logo.png", FileType::Png);
@@ -656,6 +853,16 @@ mod tests {
         test_detect_match("files/ex.tar.zst", FileType::Zst);
     }
 
+    #[test]
+    fn test_gpg_detect() {
+        test_detect_match("files/test-db.db.gpg", FileType::Gpg);
+    }
+
+    #[test]
+    fn test_armored_gpg_detect() {
+        test_detect_match("files/test-db.db.asc", FileType::ArmoredGpg);
+    }
+
     fn test_detect<P>(path: P) -> Result<Option<FileTypeMatches>, Error>
     where
         P: AsRef<Path>,
@@ -667,6 +874,17 @@ mod tests {
         detect(buf)
     }
 
+    fn test_fast_detect<P>(path: P) -> Result<Option<FileTypeMatches>, Error>
+    where
+        P: AsRef<Path>,
+    {
+        let file = OpenOptions::new().read(true).open(path).unwrap();
+
+        let mut buf = BufReader::new(file);
+
+        detect_at_start_from_ref(&mut buf)
+    }
+
     fn test_detect_sliding<P>(path: P) -> Result<Option<FileTypeMatches>, Error>
     where
         P: AsRef<Path>,
@@ -675,7 +893,7 @@ mod tests {
 
         let mut buf = BufReader::new(file);
 
-        detect_at_end(&mut buf)
+        detect_at_end_from_ref(&mut buf)
     }
 
     fn test_detect_match<P>(path: P, file_type: FileType) -> ()
@@ -720,6 +938,35 @@ mod tests {
         assert_eq!(should_match, matches);
     }
 
+    fn test_fast_detect_match_n<P>(path: P, file_type_match: Vec<FileTypeMatch>) -> ()
+    where
+        P: AsRef<Path>,
+    {
+        let detect = test_fast_detect(path).map_err(|e| e.kind());
+        assert_eq!(true, detect.is_ok());
+        let detect_option = detect.unwrap();
+        assert_eq!(true, detect_option.is_some());
+
+        let types: HashSet<FileType> = file_type_match
+            .iter()
+            .map(|v| v.file_type.clone())
+            .collect();
+
+        let detected = detect_option.unwrap();
+        let likely_types: HashSet<FileType> =
+            detected.likely_to_be.iter().map(|v| v.clone()).collect();
+
+        assert_eq!(types, likely_types);
+
+        let should_match: HashSet<FileTypeMatch> =
+            file_type_match.iter().map(|v| v.clone()).collect();
+
+        let matches: HashSet<FileTypeMatch> =
+            detected.all_matches.iter().map(|v| v.clone()).collect();
+
+        assert_eq!(should_match, matches);
+    }
+
     fn test_detect_match_maybe<P>(path: P, file_type: FileType) -> ()
     where
         P: AsRef<Path>,
@@ -756,77 +1003,4 @@ mod tests {
 
         assert_eq!(detect, expected);
     }
-
-    #[bench]
-    fn bench_multi_detect(b: &mut Bencher) {
-        b.iter(|| {
-            let _ = detect_rar();
-            let _ = detect_zip();
-            let _ = detect_jar();
-            let _ = detect_png();
-        })
-    }
-
-    #[bench]
-    fn bench_little_tar_detect(b: &mut Bencher) {
-        b.iter(|| test_detect("files/test.tar"))
-    }
-
-    #[bench]
-    fn bench_1mb_tar_detect(b: &mut Bencher) {
-        b.iter(|| test_detect("files/test-0.tar"))
-    }
-
-    #[bench]
-    fn bench_rar_sfx_detect(b: &mut Bencher) {
-        b.iter(|| test_detect("files/hello-world.exe"))
-    }
-
-    fn detect_rar() -> Result<(), Error> {
-        let f = OpenOptions::new()
-            .read(true)
-            .open("files/hello.rar")
-            .unwrap();
-
-        let buf = BufReader::new(f);
-
-        detect(buf)?;
-        Ok(())
-    }
-
-    fn detect_png() -> Result<(), Error> {
-        let f = OpenOptions::new()
-            .read(true)
-            .open("files/rust-logo.png")
-            .unwrap();
-
-        let buf = BufReader::new(f);
-
-        detect(buf)?;
-        Ok(())
-    }
-
-    fn detect_zip() -> Result<(), Error> {
-        let f = OpenOptions::new()
-            .read(true)
-            .open("files/hello.zip")
-            .unwrap();
-
-        let buf = BufReader::new(f);
-
-        detect(buf)?;
-        Ok(())
-    }
-
-    fn detect_jar() -> Result<(), Error> {
-        let f = OpenOptions::new()
-            .read(true)
-            .open("files/hello.jar")
-            .unwrap();
-
-        let buf = BufReader::new(f);
-
-        detect(buf)?;
-        Ok(())
-    }
 }
