@@ -92,7 +92,7 @@ pub trait FileTypeMatcher {
 
 impl FileTypeMatcher for FileType {
     fn test(&self, relative_position: &RelativePosition, step: &Step, bytes: &[u8]) -> TestResult {
-        match &self {
+        match self {
             FileType::Zip => zip_test(&relative_position, step, &bytes),
             FileType::Rar => rar_test(&relative_position, &bytes),
             FileType::Rar5 => rar5_test(&relative_position, &bytes),
@@ -128,6 +128,8 @@ impl FileTypeMatcher for FileType {
             FileType::Gif => gif_test(&relative_position, step, &bytes),
             FileType::Bmp => bmp_test(&relative_position, step, &bytes),
             FileType::Iso => iso_test(&relative_position, step, &bytes),
+            FileType::Gpg => gpg_test(&relative_position, step, &bytes),
+            FileType::ArmoredGpg => armored_gpg_test(&relative_position, step, &bytes),
         }
     }
 }
@@ -138,29 +140,32 @@ const LFHS: [u8; 4] = [0x50, 0x4b, 0x03, 0x04];
 const ECDS: [u8; 4] = [0x50, 0x4b, 0x05, 0x06];
 
 fn zip_test(relative_position: &RelativePosition, step: &Step, bytes: &[u8]) -> TestResult {
-    return if *relative_position == RelativePosition::Start {
-        if bytes.len() < 4 || *step != Step::Small {
-            TestResult::NotMatched
-        } else {
-            let header = &bytes[..4];
-            if header == LFHS {
-                TestResult::Maybe
-            } else {
+    match *relative_position {
+        RelativePosition::Start => {
+            if bytes.len() < 4 || *step != Step::Small {
                 TestResult::NotMatched
+            } else {
+                let header = &bytes[..4];
+                if header == LFHS {
+                    TestResult::Maybe
+                } else {
+                    TestResult::NotMatched
+                }
             }
         }
-    } else {
-        if bytes.len() < 4 {
-            TestResult::NotMatched
-        } else {
-            let signature = &bytes[..4];
-            return if signature == ECDS {
-                TestResult::Matched
+        RelativePosition::End => {
+            if bytes.len() < 4 {
+                TestResult::NotMatched
             } else {
-                TestResult::Maybe
-            };
+                let signature = &bytes[..4];
+                return if signature == ECDS {
+                    TestResult::Matched
+                } else {
+                    TestResult::Maybe
+                };
+            }
         }
-    };
+    }
 }
 
 const RAR_4_MARKER: [u8; 7] = [0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x00];
@@ -585,6 +590,33 @@ fn iso_test(relative_position: &RelativePosition, step: &Step, bytes: &[u8]) ->
     return TestResult::NotMatched;
 }
 
+const ARMORED_GPG_SIGNATURE_1: [u8; 27] = [
+    0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x42, 0x45, 0x47, 0x49, 0x4e, 0x20, 0x50, 0x47, 0x50, 0x20, 0x4d,
+    0x45, 0x53, 0x53, 0x41, 0x47, 0x45, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d,
+];
+fn armored_gpg_test(relative_position: &RelativePosition, step: &Step, bytes: &[u8]) -> TestResult {
+    if *relative_position == RelativePosition::Start && *step == Step::Small {
+        if bytes.len() >= 27 {
+            return slice_test(&bytes, &ARMORED_GPG_SIGNATURE_1);
+        }
+    }
+
+    return TestResult::NotMatched;
+}
+
+const GPG_SIGNATURE_1: [u8; 2] = [0x85, 0x03];
+fn gpg_test(relative_position: &RelativePosition, step: &Step, bytes: &[u8]) -> TestResult {
+    if *relative_position == RelativePosition::Start && *step == Step::Small {
+        if bytes.len() >= 4 {
+            if bytes[0] == GPG_SIGNATURE_1[0] && bytes[3] == GPG_SIGNATURE_1[1] {
+                return TestResult::Matched;
+            }
+        }
+    }
+
+    return TestResult::NotMatched;
+}
+
 fn slice_test(bytes: &[u8], matching: &[u8]) -> TestResult {
     if bytes.len() >= matching.len() {
         if &bytes[..matching.len()] == matching {
